@@ -3,6 +3,8 @@ import string
 import keyword
 import struct
 
+py_ssize_t_clean = False
+
 class ArgTypeError(Exception):
     pass
 
@@ -93,7 +95,7 @@ class NoneArg(ArgType):
 
 class StringArg(ArgType):
     def write_param(self, ptype, pname, pdflt, pnull, info):
-        if pdflt:
+        if pdflt != None:
             if pdflt != 'NULL': pdflt = '"' + pdflt + '"'
             info.varlist.add('char', '*' + pname + ' = ' + pdflt)
         else:
@@ -128,7 +130,10 @@ class UCharArg(ArgType):
             info.varlist.add('guchar', '*' + pname + ' = "' + pdflt + '"')
         else:
             info.varlist.add('guchar', '*' + pname)
-        info.varlist.add('int', pname + '_len')
+        if py_ssize_t_clean:
+            info.varlist.add('Py_ssize_t', pname + '_len')
+        else:
+            info.varlist.add('int', pname + '_len')
         info.arglist.append(pname)
         if pnull:
             info.add_parselist('z#', ['&' + pname, '&' + pname + '_len'],
@@ -574,8 +579,12 @@ class BoxedArg(ArgType):
                '    return pyg_boxed_new(%(typecode)s, %(ret)s, %(copy)s, TRUE);'
     def write_return(self, ptype, ownsreturn, info):
         if ptype[-1] == '*':
-            info.varlist.add(self.typename, '*ret')
+            decl_type = self.typename
             ret = 'ret'
+            if ptype[:6] == 'const-':
+                decl_type = 'const ' + self.typename
+                ret = '(%s*) ret' % (self.typename,)
+            info.varlist.add(decl_type, '*ret')
         else:
             info.varlist.add(self.typename, 'ret')
             ret = '&ret'
