@@ -24,7 +24,6 @@
 #endif
 
 #include "pygtktreemodel.h"
-#include <Python.h>
 #include "pygtk-private.h"
 
 /* define this to print out debug messages */
@@ -222,11 +221,10 @@ pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret;
-
+    guint ret = 0;
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -237,18 +235,14 @@ pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_flags", "");
     Py_DECREF(self);
     if (py_ret) {
-	guint ret = PyInt_AsLong(py_ret);
-
+	ret = PyInt_AsLong(py_ret);
 	Py_DECREF(py_ret);
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return ret;
     } else {
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return 0;
     }
+    
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gint
@@ -256,12 +250,12 @@ pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret;
-
+    gint ret = 0;
+    
     g_return_val_if_fail(tree_model != NULL, 0);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -272,18 +266,13 @@ pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_n_columns", "");
     Py_DECREF(self);
     if (py_ret) {
-	gint ret = PyInt_AsLong(py_ret);
-
+	ret = PyInt_AsLong(py_ret);
 	Py_DECREF(py_ret);
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return ret;
     } else {
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return 0;
     }
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static GType
@@ -291,11 +280,11 @@ pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret;
-
+    GType ret = G_TYPE_INVALID;
+    
     g_return_val_if_fail(tree_model != NULL, G_TYPE_INVALID);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), G_TYPE_INVALID);
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -307,18 +296,13 @@ pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 				 "(i)", index);
     Py_DECREF(self);
     if (py_ret) {
-	GType ret = pyg_type_from_object(py_ret);
-
+	ret = pyg_type_from_object(py_ret);
 	Py_DECREF(py_ret);
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return ret;
     } else {
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return G_TYPE_INVALID;
     }
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gboolean
@@ -327,14 +311,14 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 {
     PyGILState_STATE state;
     PyObject *self, *py_path, *py_ret;
-
+    gboolean ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(path != NULL, FALSE);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -355,23 +339,18 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return TRUE;
+	    ret = TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return FALSE;
 	}
     } else {
 	PyErr_Print();
 	iter->user_data = NULL;
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return FALSE;
     }
+
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static GtkTreePath *
@@ -379,13 +358,13 @@ pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
-
+    GtkTreePath *path = NULL;
+    
     g_return_val_if_fail(tree_model != NULL, NULL);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), NULL);
     g_return_val_if_fail(VALID_ITER(iter, tree_model), NULL);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -401,21 +380,18 @@ pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
                                  py_iter);
     Py_DECREF(self);
     if (py_ret) {
-	GtkTreePath *path = pygtk_tree_path_from_pyobject(py_ret);
+	path = pygtk_tree_path_from_pyobject(py_ret);
 
 	if (!path)
 	    g_warning("could not convert return value of get_path() to "
 		      "a GtkTreePath");
 	Py_DECREF(py_ret);
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return path;
     } else {
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return NULL;
     }
+    
+    pyg_gil_state_release(state);
+    return path;
 }
 
 static void
@@ -429,8 +405,7 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
     g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
     g_return_if_fail(VALID_ITER(iter, tree_model));
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -457,8 +432,7 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
     } else {
 	PyErr_Print();
     }
-    PyGILState_Release(state);
-    pyg_unblock_threads();
+    pyg_gil_state_release(state);
 }
 
 static gboolean
@@ -466,13 +440,13 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
-
+    gboolean ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(VALID_ITER(iter, tree_model), FALSE);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -494,23 +468,18 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return TRUE;
+	    ret = TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return FALSE;
 	}
     } else {
         iter->user_data = NULL;
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return FALSE;
     }
+
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gboolean
@@ -520,14 +489,14 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_parent = Py_None;
-
+    gboolean ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(parent == NULL || parent->stamp == PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp, FALSE);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -548,23 +517,18 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return TRUE;
+	    ret = TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return FALSE;
     }
+    
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gboolean
@@ -573,13 +537,13 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model,
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
-
+    gboolean ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(VALID_ITER(iter, tree_model), FALSE);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -595,18 +559,15 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model,
 				 "(O)", py_iter);
     Py_DECREF(self);
     if (py_ret) {
-	gboolean ret = PyObject_IsTrue(py_ret);
+	ret = PyObject_IsTrue(py_ret);
 
 	Py_DECREF(py_ret);
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return ret;
     } else {
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return FALSE;
     }
+    
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gint
@@ -615,13 +576,13 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model,
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
-
+    guint ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, 0);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
     g_return_val_if_fail(iter == NULL || iter->stamp == PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp, 0);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -636,18 +597,15 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model,
 				 "(O)", py_iter);
     Py_DECREF(self);
     if (py_ret) {
-	gint ret = PyInt_AsLong(py_ret);
+	ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return ret;
     } else {
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return 0;
     }
+    
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gboolean
@@ -657,14 +615,14 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_parent = Py_None;
-
+    gboolean ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(parent == NULL || parent->stamp == PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp, FALSE);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -685,23 +643,18 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return TRUE;
+	    ret = TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return FALSE;
     }
+    
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static gboolean
@@ -711,14 +664,14 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 {
     PyGILState_STATE state;
     PyObject *self, *py_ret, *py_child = Py_None;
-
+    gboolean ret = FALSE;
+    
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(VALID_ITER(child, tree_model), FALSE);
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -739,23 +692,18 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return TRUE;
+	    ret = TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
-	    PyGILState_Release(state);
-	    pyg_unblock_threads();
-	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
-	PyGILState_Release(state);
-	pyg_unblock_threads();
-	return FALSE;
     }
+    
+    pyg_gil_state_release(state);
+    return ret;
 }
 
 static void
@@ -768,8 +716,7 @@ pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
     g_return_if_fail(VALID_ITER(iter, tree_model));
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -794,8 +741,7 @@ pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
     }
     Py_DECREF(self);
 
-    PyGILState_Release(state);
-    pyg_unblock_threads();
+    pyg_gil_state_release(state);
 }
 
 static void
@@ -808,8 +754,7 @@ pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
     g_return_if_fail(VALID_ITER(iter, tree_model));
 
-    pyg_block_threads();
-    state = PyGILState_Ensure();
+    state = pyg_gil_state_ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -834,8 +779,7 @@ pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
     }
     Py_DECREF(self);
 
-    PyGILState_Release(state);
-    pyg_unblock_threads();
+    pyg_gil_state_release(state);
 }
 
 void
@@ -856,3 +800,42 @@ pygtk_generic_tree_model_iter_is_valid(PyGtkGenericTreeModel *tree_model,
 
     return VALID_ITER(iter, tree_model);
 }
+
+PyObject *
+pygtk_generic_tree_model_get_user_data(PyGtkGenericTreeModel *tree_model,
+                                       GtkTreeIter *iter)
+{
+    g_return_val_if_fail(tree_model != NULL, NULL);
+
+    if (VALID_ITER(iter, tree_model)) {
+        /* Py_INCREF and NULL checking is done at _wrap_*() level. */
+        return iter->user_data;
+    }
+    else {
+        g_warning("iter is not valid for the tree model");
+        return NULL;
+    }
+}
+
+GtkTreeIter
+pygtk_generic_tree_model_create_tree_iter(PyGtkGenericTreeModel *tree_model,
+                                          PyObject *user_data)
+{
+    GtkTreeIter  iter;
+
+    if (tree_model != NULL) {
+	iter.user_data = user_data;
+	iter.stamp = PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp;
+
+	/* Otherwise, caller is supposed to hold a reference somewhere. */
+	if (PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references)
+	    Py_INCREF(user_data);
+    }
+    else {
+	/* FIXME: I guess this is still not enough. */
+	iter.user_data = NULL;
+	iter.stamp = 0;
+    }
+
+    return iter;
+}
